"""
Abstract base class for defining ML Models.

DO NOT CHANGE THIS CODE DURING IMPLEMENTATION.
"""
import os, sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))


from abc import ABC, abstractmethod


class BaseAdapter(ABC):
	"""
	Abstract class to be implemented.
	"""

	@abstractmethod
	def send_request(self):
		"""
		Used by the client(s) to send images to the server.

		args:
			datetime: Used as an ID of the request. Should be datetime in utc format. e.g. datetime.datetime.now(timezone.utc)
			image_bytes_list: list of images in bytes.
			tool_name: name of the ML service.
			params: a dictionary of parameters as needed by the service.
		"""

		raise NotImplementedError



	@abstractmethod
	def send_response(self):
		"""
		Sends back the response from the server. Normally, the response will be converted to a native Python data type.
		"""
		
		raise NotImplementedError





from datetime import datetime, timezone
from external_vision_gateway_adapter.server_client_adapter import ServerClientAdapter

def load_image(image_path):
	"""Read an image file as binary data."""
	with open(image_path, "rb") as f:
		return f.read()


def send_image(image_path):
	# Load image data
	image_data = [load_image(img_path) for img_path in image_path]

	datetime_now = datetime.now(timezone.utc)

	response = ServerClientAdapter(port=50051, timeout=2).send_request(datetime=datetime_now, image_bytes_list=image_data, tool_name='ingrind')

	print(response)


if __name__ == '__main__':

	image_path = ["test.bmp", "test2.bmp"]  # Replace with your image file path
	send_image(image_path)


"""
Adapter for client-server(gateway) communication.
"""

import os, sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

import grpc
from google.protobuf.timestamp_pb2 import Timestamp
import resources.grpc.server_proto.external_tool_service_pb2 as external_tool_service_pb2
import resources.grpc.server_proto.external_tool_service_pb2_grpc as external_tool_service_pb2_grpc
from external_vision_gateway_adapter.base_adapter import BaseAdapter


class ServerClientAdapter(BaseAdapter):

	def __init__(self, ip="0.0.0.0", port=50051, timeout=2):

		self.ip = ip
		self.port = port
		self.timeout = timeout

	
	def send_request(self, datetime, 
					image_bytes_list, tool_name,
					params=None):
		"""
		Used by the client(s) to send images to the server.

		args:
			image_bytes: list of images in bytes. Images are expected to be in RGB format.
			tool_name: name of the ML service.
			params: additional parameters for the ML service in dict format.
		"""

		assert len(image_bytes_list) > 0, "There must be at least 1 element in the image_bytes_list"

		channel = grpc.insecure_channel(f"{self.ip}:{self.port}")
		stub = external_tool_service_pb2_grpc.ExternalToolServiceStub(channel)

		timestamp = Timestamp()
		timestamp.FromDatetime(datetime)

		request = external_tool_service_pb2.ExternalToolRequest(req_datetime=timestamp, tool_name=tool_name, payload=image_bytes_list, params=params)

		try:

			response = self.send_response(stub, request, self.timeout)
			return response

		except grpc.RpcError as e:
			print(f"gRPC Error: {e.code()} - {e.details()}")
			return e


	def send_response(self, stub, request, timeout):
		"""
		Used to convert the response from the gateway into a native python format.
		"""

		gateway_response = stub.HandleExternalToolRequest(request, timeout=timeout)

		#convert the gateway response into a python's native Dictionary type.
		result_json = {}

		for k, v in gateway_response.results.fields.items():

			result_json[k] = []

			for item in v.list_value:
				result_json[k].append(item)
		

		return result_json


		

	


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: external_tool_service.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'external_tool_service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x65xternal_tool_service.proto\x12\x07gateway\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1cgoogle/protobuf/struct.proto\"\xd4\x01\n\x13\x45xternalToolRequest\x12\x30\n\x0creq_datetime\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x11\n\ttool_name\x18\x02 \x01(\t\x12\x38\n\x06params\x18\x03 \x03(\x0b\x32(.gateway.ExternalToolRequest.ParamsEntry\x12\x0f\n\x07payload\x18\x04 \x03(\x0c\x1a-\n\x0bParamsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"=\n\x11\x45xternalToolReply\x12(\n\x07results\x18\x01 \x01(\x0b\x32\x17.google.protobuf.Struct2l\n\x13\x45xternalToolService\x12U\n\x19HandleExternalToolRequest\x12\x1c.gateway.ExternalToolRequest\x1a\x1a.gateway.ExternalToolReplyb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'external_tool_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_EXTERNALTOOLREQUEST_PARAMSENTRY']._loaded_options = None
  _globals['_EXTERNALTOOLREQUEST_PARAMSENTRY']._serialized_options = b'8\001'
  _globals['_EXTERNALTOOLREQUEST']._serialized_start=104
  _globals['_EXTERNALTOOLREQUEST']._serialized_end=316
  _globals['_EXTERNALTOOLREQUEST_PARAMSENTRY']._serialized_start=271
  _globals['_EXTERNALTOOLREQUEST_PARAMSENTRY']._serialized_end=316
  _globals['_EXTERNALTOOLREPLY']._serialized_start=318
  _globals['_EXTERNALTOOLREPLY']._serialized_end=379
  _globals['_EXTERNALTOOLSERVICE']._serialized_start=381
  _globals['_EXTERNALTOOLSERVICE']._serialized_end=489
# @@protoc_insertion_point(module_scope)



from setuptools import setup, find_packages

setup(
    name='external-vision-gateway-adapter-package',
    version='0.1.4',
    author='Ugenteraan',
    author_email='ugen@evlos.ai',
    description='Gateway Adapter to be used by clients to communicate with the gateway service.',
    packages=find_packages(),
    install_requires=[
        'grpcio',
        'grpcio-tools',
        'protobuf'
    ],
    include_package_data=True,
    python_requires='>=3.9',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)
